# -*- coding: utf-8 -*-

import wx

from outwiker.gui.tester import Tester


class TestedDialog(wx.Dialog):
    """
    Диалог, который можно тестировать через Unit Test,
    поскольку ему можно заранее установить результат
    будущего вызова метода ShowModal
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def ShowModal(self):
        result = Tester.dialogTester.runNext(self)
        if result is not None:
            event = wx.CommandEvent(wx.EVT_BUTTON.typeId, result)
            wx.PostEvent(self, event)

        return super().ShowModal()


class TestedStandardDialogMixin:
    def __init__(self, *args, **kwargs):
        self._testedValue = None
        super().__init__(*args, **kwargs)

    def ShowModal(self):
        result = Tester.dialogTester.runNext(self)
        if result is not None:
            return result

        return super().ShowModal()

    def SetDataForTest(self, value):
        self._testedValue = value


class TestedFileDialog(TestedStandardDialogMixin, wx.FileDialog):
    """
    Диалог для выбора файлов, который можно тестировать
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def GetPath(self):
        return (self._testedValue if self._testedValue is not None
                else super().GetPath())


class TestedColourDialog(TestedStandardDialogMixin, wx.ColourDialog):
    """
    Диалог для выбора цвета, который можно тестировать
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def GetColourData(self) -> wx.ColourData:
        return (self._testedValue if self._testedValue is not None
                else super().GetColourData())


class TestedSingleChoiceDialog(TestedStandardDialogMixin,
                               wx.SingleChoiceDialog):
    """
    Диалог для выбора строки из списка, который можно тестировать
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def GetStringSelection(self) -> str:
        return (self._testedValue if self._testedValue is not None
                else super().GetStringSelection())


class TestedDirDialog(TestedStandardDialogMixin, wx.DirDialog):
    """
    Testable select directory dialog 
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def GetPath(self):
        return (self._testedValue if self._testedValue is not None
                else super().GetPath())
