# -*- coding: utf-8 -*-

import re

from pyparsing import Regex

from outwiker.core.thumbexception import ThumbException
from outwiker.core.defines import PAGE_ATTACH_DIR
from outwiker.core.htmlformatter import HtmlFormatter
from outwiker.core.cssclasses import CSS_WIKI
from .pagethumbmaker import PageThumbmaker
from ..wikiconfig import WikiConfig


class ThumbnailFactory:
    """
    Класс для создания токена ThumbnailToken
    """
    @staticmethod
    def make(parser):
        return ThumbnailToken(parser).getToken()


class ThumbnailToken:
    """
    Класс, содержащий все необходимое для разбора и создания превьюшек
    картинок на вики-странице
    """

    def __init__(self, parser):
        self.parser = parser
        self.thumbmaker = PageThumbmaker()
        self._html_formatter = HtmlFormatter([CSS_WIKI])

    def getToken(self):
        result = Regex(r"""%\s*?
                        (?:
                            (?:thumb\s+)?
                            (?:width\s*?=\s*?(?P<width>\d+)
                            |height\s*?=\s*?(?P<height>\d+)
                            |maxsize\s*?=\s*?(?P<maxsize>\d+))\s*?
                            (?:px)?
                            |thumb\s*?
                        )\s*?
                        %\s*?
                        Attach:(?P<quote>["']?)(?P<fname>.*?\.(?:jpe?g|bmp|gif|tiff?|png|webp))(?P=quote)\s*?%%""",
                       re.IGNORECASE | re.VERBOSE)
        result = result.setParseAction(self.__convertThumb)("thumbnail")
        return result

    def __convertThumb(self, s, l, t):
        if t["width"] is not None:
            size = int(t["width"])
            func = self.thumbmaker.createThumbByWidth

        elif t["height"] is not None:
            size = int(t["height"])
            func = self.thumbmaker.createThumbByHeight

        elif t["maxsize"] is not None:
            size = int(t["maxsize"])
            func = self.thumbmaker.createThumbByMaxSize

        else:
            config = WikiConfig(self.parser.config)
            size = config.thumbSizeOptions.value
            func = self.thumbmaker.createThumbByMaxSize

        fname = t["fname"]

        try:
            thumb = func(self.parser.page, fname, size)
        except (ThumbException, IOError):
            text = _("Can't create thumbnail for <b>\"{}\"</b>").format(fname)
            return self._html_formatter.error(text)

        return '<a href="{}/{}"><img src="{}"/></a>'.format(PAGE_ATTACH_DIR, fname, thumb.replace("\\", "/"))
